@@ -17,6 +17,41 @@ export class RepositoriesService {
     }
 
     public createHTTPS({
+        type,
+        name,
+        url,
+        username,
+        password,
+        tlsClientCertData,
+        tlsClientCertKey,
+        insecure,
+        enableLfs,
+        proxy,
+        project,
+        forceHttpBasicAuth,
+        enableOCI
+    }: {
+        type: string;
+        name: string;
+        url: string;
+        username: string;
+        password: string;
+        tlsClientCertData: string;
+        tlsClientCertKey: string;
+        insecure: boolean;
+        enableLfs: boolean;
+        proxy: string;
+        project?: string;
+        forceHttpBasicAuth?: boolean;
+        enableOCI: boolean;
+    }): Promise<models.Repository> {
+        return requests
+            .post('/repositories')
+            .send({type, name, repo: url, username, password, tlsClientCertData, tlsClientCertKey, insecure, enableLfs, proxy, project, forceHttpBasicAuth, enableOCI})
+            .then(res => res.body as models.Repository);
+    }
+
+    public updateHTTPS({
         type,
         name,
         url,
@@ -42,7 +77,7 @@ export class RepositoriesService {
         project?: string;
     }): Promise<models.Repository> {
         return requests
-            .post('/repositories')
+            .put(`/repositories/${encodeURIComponent(url)}`)
             .send({type, name, repo: url, username, password, tlsClientCertData, tlsClientCertKey, insecure, enableLfs, proxy, project})
             .then(res => res.body as models.Repository);
     }
@@ -121,6 +156,34 @@ export class RepositoriesService {
             .then(res => res.body as models.Repository);
     }
 
+    public createGoogleCloudSource({
+        type,
+        name,
+        url,
+        gcpServiceAccountKey,
+        proxy,
+        project
+    }: {
+        type: string;
+        name: string;
+        url: string;
+        gcpServiceAccountKey: string;
+        proxy: string;
+        project?: string;
+    }): Promise<models.Repository> {
+        return requests
+            .post('/repositories')
+            .send({
+                type,
+                name,
+                repo: url,
+                gcpServiceAccountKey,
+                proxy,
+                project
+            })
+            .then(res => res.body as models.Repository);
+    }
+
     public delete(url: string): Promise<models.Repository> {
         return requests
             .delete(`/repositories/${encodeURIComponent(url)}`)
